# This function is written by Sirus Dehdari (sirus.dehdari@ne.su.se), PhD candidate
# in Economics at Stockholm University and visiting scholar at Harvard University.
# Latest version: 4/26/2015

### The function creates a "visual" RDD, with regression lines on both sides
### of the cut-off, based on separate regressions using data on each side.
### When controls are added, the residuals from a regression of the dependent
### variable on the controls is plottet against the running variable.

### For now, it is important that the running variable is already centered
### around zero. I will update the function so that one instead sets the cut-off
### value and the running variable will then be centered in the function.

### Arguments
### - fdata: your data set
### - dep: name of the dependent variable
### - run: name of the running variable
### - control: name(s) of the control variable(s). Default is set at no controls
### - band: bandwidth, in whatever metric the running variable is in
### - bin: the binsize of each point in the plot. Default is set at 1
### - poly: highest degree of the polynomial


##### THE FUNCTION BEGINS HERE
rddplot <- function(fdata, dep, run, control = "no", band, bin = 1, poly = 1, pshape = 1, psize = 1, pcol = "gray",
                    lsize = 1, lcol = "black", mshape = 19, msize = 1, mcol = "black"){
  library(ggplot2)
  fdata <- fdata[abs(fdata[,run]) <band,]
  ### Dividing the data into left and right side of the cut-off
  pdata <- fdata[fdata[,run] > 0,]
  ndata <- fdata[fdata[,run] <= 0,]
  
  # Creating an expression for the regression model:
  lhs <- paste(dep,"~",sep = "")
  rhs = paste(paste("I(",run,"^",seq(1:poly),")",sep = ""),collapse = "+")
  regmod = paste(lhs,rhs, sep = "")
  
  ### Starting with the side to the right of the threshold, a matrix is created for 
  ### the mean values of the dependent variable for each
  ### bin, and the number of bins is a function of the bandwidth and the binsize
  posl <- matrix(0,nrow = round((band*2/bin)/2), ncol = 2)
  
  ### Based on the number of bins, the increment (ink) is decided and a 
  ### sequence (sek) is created, from 0 to the end of the bandwidth, increasing by
  ### inc. Using this sequence, the lower and upper bound for each bin is used
  ### to subset the data, where the mean of the dependent variable is estimated
  ### using the subsetted data
  inc <- band/dim(posl)[1]
  sek <- seq(0, band, by = inc)
  for (i in 1:dim(posl)[1]){
    l <- sek[i]
    u <- sek[i+1]
    sub <- fdata[fdata[,run] > l & fdata[,run]  <= u,]
    posl[i,1] <- mean(sub[,dep], na.rm = TRUE)
    posl[i,2] <- (u+l)/2
  }
  rm(inc, sek)
  
  ### The same thing is done for the left side of the cut-off
  negl <- matrix(0,nrow = round((band*2/bin)/2), ncol = 2)
  inc <- band/dim(negl)[1]
  sek <- seq(-band, 0, by = inc)
  for (i in 1:dim(negl)[1]){
    l <- sek[i]
    u <- sek[i+1]
    sub <- fdata[fdata [,run]> l & fdata[,run]  <= u,]
    negl[i,1] <- mean(sub[,dep], na.rm = TRUE)
    negl[i,2] <- (u+l)/2
  }
  plotd <- as.data.frame(rbind(negl, posl))
  colnames(plotd) <- c(dep,run)
  
  ### Creating prediction for the regression line, one on each side of the threshold
  tempd_nona <- na.omit(pdata[, c(run,dep)]) # Take out non-NA values of the run. var. and the dep. var. from the positive data
  pos_pred <- predict(lm(regmod, pdata)) # Retrieve predicted values based on the model for the positive data
  pos_preddata <- cbind(tempd_nona, pos_pred, NA) # Put together predictions and positive data (dep and run)
  colnames(pos_preddata)[3:4] <- c("pos_pred", "neg_pred")
  rm(tempd_nona, pos_pred)
  tempd_nona <- na.omit(ndata[, c(run,dep)])
  neg_pred <- predict(lm(regmod, ndata))
  neg_preddata <- cbind(tempd_nona, NA, neg_pred)
  colnames(neg_preddata)[3:4] <- c("pos_pred", "neg_pred")
  rm(tempd_nona, neg_pred)
  
  final_data <- rbind(neg_preddata,pos_preddata)
  
  ### The plot is created using the positions of the bins and the mean of the
  ### dependent variable (or the residuals in case of added controls). To this,
  ### regression lines are added together with a vertical line at the cut-off = 0.
  
  p <- ggplot(final_data, aes(y = get(dep), x = get(run)))
  
  pp <- p + geom_point(colour = pcol, shape = pshape, size = psize) + 
    geom_point(data = plotd, colour = mcol, shape = mshape, size = msize) + 
    geom_line(aes(y = pos_pred), size = lsize, colour = lcol) + 
    geom_line(aes(y = neg_pred), size = lsize, colour = lcol)
  
  
  return(pp)
}
##### THE FUNCTION ENDS HERE